package qa.qcri.prepocressor.operations;

import java.util.LinkedList;
import java.util.List;

import qa.qcri.prepocressor.datastructures.Individual;
import qa.qcri.prepocressor.io.Logger;
import qa.qcri.prepocressor.ui.Configuration;

/*
 This file is part of the prepocressor toolkit.

 Copyright (c) 2015, QCRI a member of Qatar Foundation. All Rights Reserved
 Felix Stahlberg

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

/**
 * This class implements the potatoPeelingTextLine operation.
 * 
 * @author Felix Stahlberg
 */
public class PotatoPeelingTextLineOperation extends Operation {

	/*
	 * (non-Javadoc)
	 * 
	 * @see Operation#Operation(String)
	 */
	public PotatoPeelingTextLineOperation(String name) {
		super(name);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Operation#createConfiguration()
	 */
	@Override
	protected Configuration createConfiguration() {
		return new Configuration() {
			@Override
			protected void addDefaultParameters() {
				;
			}

			@Override
			protected String getDescription() {
				return "NOT IMPLEMENTED YET! This operation aims to correct curved baselines "
						+ "in Arabic script (condition: convexity). It fits the "
						+ "largest convex set of background pixels above the letters "
						+ "and adjusts each column accordingly. Finding a largest "
						+ "convex subset of a non-convex set is know as potato "
						+ "peeling problem. Note that while general potato peeling "
						+ "algorithms are computational complex we can exploit our "
						+ "special case (i.e. that all points on the upper canvas "
						+ "border must belong to the set). Therefore, we can just "
						+ "iteratively remove points which cause non-convexity ordered";
			}
		};
	}

	@Override
	protected List<Individual> processIndividual(Individual indy) {
		Logger.getSingleton().fatalError(
				"potatoPeelingCorrection not implmented yet!");
		List<Individual> children = new LinkedList<Individual>();
		children.add(indy);
		return children;
	}
}
